/* Unhide yjesus@security-projects.com */

/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// Needed for unistd.h to declare getpgid() and others
#define _XOPEN_SOURCE 500

// Needed for sched.h to declare sched_getaffinity()
#define _GNU_SOURCE

#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include <wait.h>
#include <sys/resource.h>
#include <errno.h>
#include <dirent.h>
#include <sched.h>
#include <sys/types.h>
#include <signal.h>
#include <stdlib.h>
#include <sys/sysinfo.h>
#include <fcntl.h>
#include <pthread.h>
#include <sys/syscall.h>
#include <ctype.h>


// we are looking only for real process not thread and only one by one
#define COMMAND "ps --no-header -p %i o pid"
// we are looking for session ID one by one
#define SESSION "ps --no-header -s %i o sess"
// We are looking for group ID one by one
// but ps can't select by pgid
#define PGID "ps --no-header -eL o pgid"
// We are looking for all processes even threads
#define THREADS "ps --no-header -eL o lwp"
// for sysinfo scanning, fall back to old command, as --no-header seems to create
// an extra process/thread
#define SYS_COMMAND "ps -eL o lwp"
// an extra process/thread
#define REVERSE "ps --no-header -eL o lwp,cmd"

// Define mask for the checks to do in checkps
#define PS_PROC			0x00000001
#define PS_THREAD			0x00000002
#define PS_MORE			0x00000004

// Define test number
#define TST_NONE			0
#define TST_PROC			1
#define TST_DIR			2
#define TST_SYS			3
#define TST_BRUTE			4
#define TST_REVERSE		5
#define TST_QUICK			6

// boolean value
#define FALSE			0
#define TRUE			1


// sysctl kernel.pid_max
int maxpid = 32768;

// For Threads sync
int tid ;

// our own PID
pid_t mypid ;

// options
int verbose = FALSE;
int morecheck = FALSE;

void *funcionThread (void *parametro) {

	tid = (pid_t) syscall (SYS_gettid);
	return(&tid) ;
};


void get_max_pid(int* newmaxpid) {
	char path[]= "/proc/sys/kernel/pid_max";
	pid_t tmppid = 0;
	FILE* fd= fopen(path,"r");
	if(!fd) {
		printf("[*] Error: cannot get current maximum PID: %s\n", strerror(errno));
		return;
	}


	if((fscanf(fd, "%d", &tmppid) != 1) || tmppid < 1) {
		printf("[*] cannot get current maximum PID: Error parsing %s format\n", path);
		return;
	} else {
		*newmaxpid = tmppid;
	}
	fclose(fd) ;
}


int checkps(int tmppid, int checks) {

	int ok = 0;
	char pids[30];

	char compare[100];
	char command[60];


	FILE *fich_tmp ;

//	printf("in --> checkps\n");   // DEBUG

// The compare string is the same for all test
	sprintf(compare,"%i\n",tmppid);

	if (PS_PROC == (checks & PS_PROC)) {
		sprintf(command,COMMAND,tmppid) ;

		fich_tmp=popen (command, "r") ;
		if (fich_tmp == NULL) {
			if(TRUE == verbose) {
				printf("Warning : popen failed while ps checking pid %d (memory, or something set errno: %s)\n", tmppid, strerror(errno));
			}
			return(0);
		}

//		while (!feof(fich_tmp) && ok == 0) {
		{
			char* tmp_pids = pids;

			fgets(pids, 30, fich_tmp);
			pids[29] = 0;

//			printf("pids = %s\n", pids);   // DEBUG
			while( *tmp_pids == ' ' && tmp_pids <= pids+29) {
				tmp_pids++;
			}

			if (strncmp(tmp_pids, compare, 30) == 0) {ok = 1;}

		}

		if (fich_tmp != NULL)
			pclose(fich_tmp);

		if (1 == ok) return(ok) ;	 // pid is found, no need to go further
	}

	if (PS_THREAD == (checks & PS_THREAD)) {
		FILE *fich_thread ;

		fich_thread=popen (THREADS, "r") ;
		if (fich_thread == NULL) {
			if(TRUE == verbose) {
				printf("Warning : popen failed while thread checking pid %d (memory, or something set errno: %s)\n", tmppid, strerror(errno));
			}
			return(0);
		}

		while (!feof(fich_thread) && ok == 0) {
			char* tmp_pids = pids;

			fgets(pids, 30, fich_thread);
			pids[29] = 0;

//			printf("   threads = %s\n", pids);   // DEBUG
			while( *tmp_pids == ' ' && tmp_pids <= pids+29) {
				tmp_pids++;
			}

			if (strncmp(tmp_pids, compare, 30) == 0) {ok = 1;}


		}
		if (fich_thread != NULL)
			pclose(fich_thread);

		if (1 == ok) return(ok) ;	 // thread is found, no need to go further
	}

	if (PS_MORE == (checks & PS_MORE)) {

		FILE *fich_session ;

		sprintf(command,SESSION,tmppid) ;

		fich_session=popen (command, "r") ;
		if (fich_session == NULL) {
			printf("Warning : popen failed while session checking pid %d (memory, or something set errno: %s)\n", tmppid, strerror(errno));
			return(0);
		}


		while (!feof(fich_session) && ok == 0) {
			char* tmp_pids = pids;

			fgets(pids, 30, fich_tmp);
			pids[29] = 0;

			while( *tmp_pids == ' ' && tmp_pids <= pids+29) {
				tmp_pids++;
			}

			if (strncmp(tmp_pids, compare, 30) == 0) {ok = 1;}

		}

		pclose(fich_session);

		if (1 == ok) return(ok) ;	 // session is found, no need to go further

		FILE *fich_pgid ;

		fich_pgid=popen (PGID, "r") ;
		if (fich_pgid == NULL) {
			if(TRUE == verbose) {
				printf("Warning : popen failed while pgid checking pid %d (memory, or something set errno: %s)\n", tmppid, strerror(errno));
			}
			return(0);
		}

		while (!feof(fich_pgid) && ok == 0) {
			char* tmp_pids = pids;

			fgets(pids, 30, fich_pgid);
			pids[29] = 0;

			while( *tmp_pids == ' ' && tmp_pids <= pids+29) {
				tmp_pids++;
			}

			if (strncmp(tmp_pids, compare, 30) == 0) {ok = 1;}

		}

		pclose(fich_pgid);

	}
	return ok;
}

void printbadpid (int tmppid) {

	int statuscmd ;
	char cmd[100] ;
	struct stat buffer;
	FILE *cmdfile ;
	char cmdcont[1000];
	int cmdok = 0;

	printf ("Found HIDDEN PID: %i\n", tmppid) ;

	sprintf(cmd,"/proc/%i/cmdline",tmppid);

	statuscmd = stat(cmd, &buffer);
//	statuscmd = 0 ;  // DEBUG

	if (statuscmd == 0) {
		cmdfile=fopen (cmd, "r") ;

		if (cmdfile != NULL) {

			while (!feof (cmdfile)) {

				if (NULL != fgets (cmdcont, 1000, cmdfile)) {
					cmdok = 1;
					printf ("Command: %s\n\n", cmdcont);
				}
			}
			fclose(cmdfile);
		}
	}
	if (0 == cmdok) {			// fallback : try to readlink the exe
		ssize_t length ;

		sprintf(cmd,"/proc/%i/exe",tmppid);
		statuscmd = stat(cmd, &buffer);
		printf("%s",cmd) ; //DEBUG
		if (statuscmd == 0) {
			length = readlink(cmd, cmdcont, 1000) ;
//			printf("\t%0d\n",(int)length) ; //DEBUG
			if (-1 != length) {
				cmdcont[length] = 0;   // terminate the string
				printf ("Exe: %s\n\n", cmdcont);
			}
		}
		else {
			printf("  ... maybe a transitory process\n");
		}
	}
}

void checkproc() {

	int procpids ;
	int statusprocbefore, statusprocafter;
	struct stat buffer;

	printf ("[*]Searching for Hidden processes through /proc stat scanning\n\n") ;

	for ( procpids = 1; procpids <= maxpid; procpids = procpids +1 ) {

		char directory[100] ;

		// avoid ourselves
		if (procpids == mypid) {
			continue;
		}
		sprintf(directory,"/proc/%d",procpids);

		statusprocbefore = stat(directory, &buffer) ;
		if (statusprocbefore != 0) {
			continue;
		}

		if(checkps(procpids,PS_PROC | PS_THREAD)) {
			continue;
		}

		statusprocafter = stat(directory, &buffer) ;
		if (statusprocafter != 0) {
			continue;
		}

		printbadpid(procpids);
	}
}

void checkchdir() {

	int procpids ;
	int statusdir;
	char curdir[PATH_MAX] ;
//	char scratch[PATH_MAX] ;	// DEBUG
//	int count = 0;		//DEBUG

	printf ("[*]Searching for Hidden processes through /proc chdir scanning\n\n") ;
	// get the path where Unhide is ran from.
	getcwd(curdir, PATH_MAX);

	for ( procpids = 1; procpids <= maxpid; procpids = procpids +1 ) {

		char directory[100] ;

		// avoid ourselves
		if (procpids == mypid) {
			continue;
		}

		sprintf(directory,"/proc/%d",procpids);

		statusdir = chdir(directory) ;
		// the directory doesn't exist continue with the next one
		if (statusdir != 0) {
			continue;
		}
		if (morecheck == TRUE) {
			// find process group ID (the master thread) by reading the status file of the current dir
			FILE *fich_tmp ;
			int found_tgid = FALSE;
			char line[128] ;
			char* tmp_pids = line;
			char* end_pid;
			char new_directory[100] ;

//			printf("directory = '%s'\n", directory);	// DEBUG
//			getcwd(scratch, PATH_MAX);						// DEBUG
//			printf("CWD = '%s'\n", scratch);				// DEBUG

			fich_tmp=fopen("status", "r") ;
			if (fich_tmp == NULL) {
				if(TRUE == verbose) {
					printf("Warning : can't open status file for process: %d)\n", procpids);
				}
				continue ; // next process
			}
			while ((FALSE == found_tgid) && (!feof (fich_tmp))) {

				if (NULL != fgets (line, 128, fich_tmp)) {
					line[127] = 0;
					if (0 == strncmp (line, "Tgid:", 5)) {
						found_tgid = TRUE;
					}
				}
			}
			fclose(fich_tmp);

			if (TRUE == found_tgid) {
				tmp_pids = line + 5;
				while( ((*tmp_pids == ' ') || (*tmp_pids == '\t'))  && (tmp_pids <= line+127)) {
					tmp_pids++;
				}
//				printf("tmp_pids2 = '%s'\n", tmp_pids);   // DEBUG
				end_pid = tmp_pids;
				while( isdigit(*end_pid) && end_pid <= line+127) {
					end_pid++;
				}
				*end_pid = 0;  // remove \n
//				if the number of threads is < to about 40 % of the number of processes,
//				the next "optimising" test actually produce a slower executable.
//				if(procpids != atoi(tmp_pids))
				{   // if the thread isn't the master thread (process)
//					count++;		// DEBUG
					sprintf(new_directory,"/proc/%s/task/%d", tmp_pids, procpids) ;
//					printf("new_dir = %s\n", new_directory);   // DEBUG
					statusdir = chdir(new_directory) ;
					if (statusdir != 0) {
					// the thread is not listed in the master thread task directory
						if(TRUE == verbose) {
							printf("Warning : Thread %s said it's in group %d but isn't listed in %s\n", tmp_pids, procpids, new_directory);
						}
					}
				}
			}
			else {
				if(TRUE == verbose) {
				printf("Warning : Can't find TGID in status file for process': %d)\n", procpids);
				}
			}
		}

		// unlock the proc directory so it can disappear if it's a transitory process
		chdir(curdir);

		if(checkps(procpids, PS_PROC | PS_THREAD)) {
			continue;
		}

		// Avoid false positive on short life process/thread
		statusdir = chdir(directory) ;
		if (statusdir != 0) {
			continue;
		}

		printbadpid(procpids);
	}
	// go back to our path
	chdir(curdir);
//	printf("Passages = %d\n", count);	// DEBUG
}


void checkreaddir() {

	int procpids ;
   DIR *procdir, *taskdir;
   struct dirent *dir, *dirproc;

	printf ("[*]Searching for Hidden thread through /proc/pid/task readdir scanning\n\n") ;

	procdir = opendir("/proc");
	if (NULL == procdir) {
		if(TRUE == verbose) {
			printf("Warning : Cannot open /proc directory !");
		}
		return ;
	}
	while ((dirproc = readdir(procdir))) {
	// As of Linux kernel 2.6 :
	// readdir directly in /proc only see process, not thread
	// because procfs voluntary hides threads to readdir
		char *directory ;
		char task[100] ;

		directory = dirproc->d_name;
		if(!isdigit(*directory)) {
			// not a process directory of /proc
			continue;
		}
//		sprintf(currentproc, "%d", directory);

		sprintf(task, "/proc/%s/task", directory) ;
//		printf("task : %s", task) ; // DEBUG
		taskdir = opendir(task);
		if (NULL == taskdir) {
			if(TRUE == verbose) {
				printf("Warning : Cannot open %s directory !", task);
			}
			return ;
		}

		while ((dir = readdir(taskdir)))
		{
			char *tmp_d_name ;
			tmp_d_name = dir->d_name;
//			printf(" thread : %s\n",tmp_d_name) ;  // DEBUG
			if (!strcmp(tmp_d_name, ".") || !strcmp(tmp_d_name, "..")) // skip parent and current dir
				continue;
			if(!isdigit(*tmp_d_name)) {
				if(TRUE == verbose) {
					printf("Warning : Not a a thread ID (%s) in %s\n", tmp_d_name, task) ;
				}
				continue;
			}
			else if (0 != strcmp(tmp_d_name, directory)) { // thread ID is not the process ID
//				printf("thread : %s\n",tmp_d_name) ;  // DEBUG
				procpids = atoi(tmp_d_name) ;
				if(checkps(procpids,PS_THREAD)) {
					continue;
				}
				printbadpid(atoi(tmp_d_name));
			}
			else {
//				printf("process : %s\n",tmp_d_name) ;  // DEBUG
				procpids = atoi(tmp_d_name) ;
				if(checkps(procpids,PS_PROC)) {
					continue;
				}
				printbadpid(atoi(tmp_d_name));
			}
		}
		closedir(taskdir);
	}
	closedir(procdir) ;
}

void checkgetpriority() {

	int syspids ;

	printf ("[*]Searching for Hidden processes through getpriority() scanning\n\n") ;

	for ( syspids = 1; syspids <= maxpid; syspids = syspids +1 ) {

		int which = PRIO_PROCESS;

		int ret;

		errno= 0 ;

		// avoid ourselves
		if (syspids == mypid) {
			continue;
		}

		ret = getpriority(which, syspids);
		if ( errno != 0) {
			continue;
		}

		if(checkps(syspids,PS_PROC | PS_THREAD)) {
			continue;
		}

		errno=0;
		ret = getpriority(which, syspids);
		if ( errno != 0) {
			continue;
		}

		printbadpid(syspids);
	}
}

void checkgetpgid() {

	int syspids ;


	printf ("[*]Searching for Hidden processes through getpgid() scanning\n\n") ;

	for ( syspids = 1; syspids <= maxpid; syspids = syspids +1 ) {

		int ret;

		errno= 0 ;

		// avoid ourselves
		if (syspids == mypid) {
			continue;
		}

		ret = getpgid(syspids);
		if ( errno != 0 ) {
			continue;
		}

		if(checkps(syspids,PS_PROC | PS_THREAD)) {
			continue;
		}

		errno=0;
		ret = getpgid(syspids);
		if ( errno != 0 ) {
			continue;
		}

		printbadpid(syspids);
	}
}


void checkgetsid() {

	int syspids ;


	printf ("[*]Searching for Hidden processes through getsid() scanning\n\n") ;

	for ( syspids = 1; syspids <= maxpid; syspids = syspids +1 ) {

		int ret;

		errno= 0 ;

		// avoid ourselves
		if (syspids == mypid) {
			continue;
		}

		ret = getsid(syspids);
		if ( errno != 0) {
			continue;
		}
		if(checkps(syspids,PS_PROC | PS_THREAD)) {
			continue;
		}
		errno=0;
		ret = getsid(syspids);
		if ( errno != 0) {
			continue;
		}

		printbadpid(syspids);

	}
}


void checksched_getaffinity() {

	int syspids;
	cpu_set_t mask;

	printf ("[*]Searching for Hidden processes through sched_getaffinity() scanning\n\n") ;

	for ( syspids = 1; syspids <= maxpid; syspids = syspids +1 ) {

		int ret;

		errno= 0 ;

		// avoid ourselves
		if (syspids == mypid) {
			continue;
		}

		ret = sched_getaffinity(syspids, sizeof(cpu_set_t), &mask);
		if (errno != 0) {
			continue;
		}
		if (checkps(syspids,PS_PROC | PS_THREAD)) {
			continue;
		}
		errno=0;
		ret = sched_getaffinity(syspids, sizeof(cpu_set_t), &mask);
		if (errno != 0) {
			continue;
		}

		printbadpid(syspids);
	}
}


void checksched_getparam() {

	int syspids;
	struct sched_param param;

	printf ("[*]Searching for Hidden processes through sched_getparam() scanning\n\n") ;


	for ( syspids = 1; syspids <= maxpid; syspids = syspids +1 ) {

		int ret;

		errno= 0 ;

		// avoid ourselves
		if (syspids == mypid) {
			continue;
		}

		ret = sched_getparam(syspids, &param);
		if ( errno != 0) {
			continue;
		}

		if(checkps(syspids,PS_PROC | PS_THREAD)) {
			continue;
		}

		errno=0;
		ret = sched_getparam(syspids, &param);
		if ( errno != 0) {
			continue;
		}

		printbadpid(syspids);

	}
}

void checksched_getscheduler() {

	int syspids ;


	printf ("[*]Searching for Hidden processes through sched_getscheduler() scanning\n\n") ;

	for ( syspids = 1; syspids <= maxpid; syspids = syspids +1 ) {

		int ret;

		errno= 0 ;

		// avoid ourselves
		if (syspids == mypid) {
			continue;
		}

		ret = sched_getscheduler(syspids);
		if ( errno != 0) {
			continue;
		}

		if(checkps(syspids,PS_PROC | PS_THREAD)) {
			continue;
		}

		errno=0;
		ret = sched_getscheduler(syspids);
		if ( errno != 0) {
			continue;
		}

		printbadpid(syspids);

	}
}

void checksched_rr_get_interval() {

	int syspids;
	struct timespec tp;

	printf ("[*]Searching for Hidden processes through sched_rr_get_interval() scanning\n\n") ;

	for ( syspids = 1; syspids <= maxpid; syspids = syspids +1 ) {

		int ret;

		errno= 0 ;

		// avoid ourselves
		if (syspids == mypid) {
			continue;
		}

		ret = sched_rr_get_interval(syspids, &tp);
		if ( errno != 0) {
			continue;
		}

		if(checkps(syspids,PS_PROC | PS_THREAD)) {
			continue;
		}

		errno=0;
		ret = sched_rr_get_interval(syspids, &tp);
		if ( errno != 0) {
			continue;
		}

		printbadpid(syspids);
	}
}

void checkkill() {

	int syspids;

	printf ("[*]Searching for Hidden processes through kill(..,0) scanning\n\n") ;

	for ( syspids = 1; syspids <= maxpid; syspids = syspids +1 ) {

		int ret;

		errno= 0 ;

		// avoid ourselves
		if (syspids == mypid) {
			continue;
		}

		ret = kill(syspids, 0);
		if ( errno != 0) {
			continue;
		}

		if(checkps(syspids,PS_PROC | PS_THREAD)) {
			continue;
		}

		errno= 0 ;
		ret = kill(syspids, 0);
		if ( errno != 0) {
			continue;
		}

		printbadpid(syspids);
	}
}

void checkallnoprocps() {

	/* compare the various system calls against each other,
	 * without invoking 'ps' or looking at /proc */

	int ret;
	int syspids;
	struct timespec tp;
	struct sched_param param;
	cpu_set_t mask;
	int found=0;
	int found_killbefore=0;
	int found_killafter=0;

	printf ("[*]Searching for Hidden processes through  comparison of results of system calls\n\n") ;

	for ( syspids = 1; syspids <= maxpid; syspids++ ) {

		// avoid ourselves
		if (syspids == mypid) {
			continue;
		}

		found=0;
		found_killbefore=0;
		found_killafter=0;

		errno=0;
		ret = kill(syspids, 0);
		if (errno == 0) found_killbefore=1;

		errno= 0 ;
		ret = getpriority(PRIO_PROCESS, syspids);
		if (errno == 0) found++;

		errno= 0 ;
		ret = getpgid(syspids);
		if (errno == 0) found++;

		errno= 0 ;
		ret = getsid(syspids);
		if (errno == 0) found++;

		errno= 0 ;
		ret = sched_getaffinity(syspids, sizeof(cpu_set_t), &mask);
		if (ret == 0) found++;

		errno= 0 ;
		ret = sched_getparam(syspids, &param);
		if (errno == 0) found++;

		errno= 0 ;
		ret = sched_getscheduler(syspids);
		if (errno == 0) found++;

		errno=0;
		ret = sched_rr_get_interval(syspids, &tp);
		if (errno == 0) found++;

		errno=0;
		ret = kill(syspids, 0);
		if (errno == 0) found_killafter=1;


		/* these should all agree, except if a process went or came in the middle */
		if (found_killbefore == found_killafter) {
			if ( ! ((found_killbefore == 0 && found == 0) ||
					  (found_killbefore == 1 && found == 7)) ) {
				printf ("Found HIDDEN PID: %i\n", syspids) ;
			}
		} /* else: unreliable */
		else {
			if(TRUE == verbose) {
				printf("Warning : syscall comparison test skipped for PID %d", syspids);
			}
		}
	}
}

void checkallquick() {

	/* compare the various system calls against each other,
	 * without invoking 'ps' or looking at /proc */

	int ret;
	int syspids;
	struct timespec tp;
	struct sched_param param;
	cpu_set_t mask;
	int found=0;
	int found_killbefore=0;
	int found_killafter=0;
	char directory[100];
	struct stat buffer;
	int statusproc, statusdir;
	char curdir[PATH_MAX] ;

	printf ("[*]Searching for Hidden processes through  comparison of results of system calls, proc, dir and ps\n\n") ;

	// get the path where Unhide is ran from.
	getcwd(curdir, PATH_MAX);

	for ( syspids = 1; syspids <= maxpid; syspids++ ) {

		// avoid ourselves
		if (syspids == mypid) {
			continue;
		}

		// printf("syspid = %d\n", syspids); //DEBUG

		found=0;
		found_killbefore=0;
		found_killafter=0;

		errno=0;
		ret = kill(syspids, 0);
		if (errno == 0) found_killbefore=1;

		errno= 0 ;
		ret = getpriority(PRIO_PROCESS, syspids);
		if (errno == 0) found++;

		errno= 0 ;
		ret = getpgid(syspids);
		if (errno == 0) found++;

		errno= 0 ;
		ret = getsid(syspids);
		if (errno == 0) found++;

		errno= 0 ;
		ret = sched_getaffinity(syspids, sizeof(cpu_set_t), &mask);
		if (ret == 0) found++;

		errno= 0 ;
		ret = sched_getparam(syspids, &param);
		if (errno == 0) found++;

		errno= 0 ;
		ret = sched_getscheduler(syspids);
		if (errno == 0) found++;

		errno=0;
		ret = sched_rr_get_interval(syspids, &tp);
		if (errno == 0) found++;

		sprintf(directory,"/proc/%d",syspids);

		statusproc = stat(directory, &buffer) ;
		if (statusproc == 0) {
			found++;
		}

		statusdir = chdir(directory) ;
		if (statusdir == 0) {
			found++;
			chdir(curdir);
		}

		if ((0 != found) || (0 != found_killbefore)) {
			if(checkps(syspids,PS_PROC | PS_THREAD)) {
				found++;
			}
		}

		errno=0;
		ret = kill(syspids, 0);
		if (errno == 0) found_killafter=1;


		/* these should all agree, except if a process went or came in the middle */
		if (found_killbefore == found_killafter) {
			if ( ! ((found_killbefore == 0 && found == 0) ||
					  (found_killbefore == 1 && found == 10)) ) {
				printbadpid(syspids);
			}
		} /* else: unreliable */
		else {
			if(TRUE == verbose) {
				printf("Warning : syscall comparison test skipped for PID %d", syspids);
			}
		}
	}
}


void checkallreverse() {

	/* verify if all thread showed by ps are also seen by others */

	int ret;
	int syspids;
	struct timespec tp;
	struct sched_param param;
	cpu_set_t mask;
	int found=0;
	int found_killbefore=0;
	int found_killafter=0;
	FILE *fich_tmp;
	char command[50];
	char read_line[1024];
	char lwp[7];
	int  index;
	char directory[100];
	struct stat buffer;
	int statusproc, statusdir;
	char curdir[PATH_MAX] ;

	printf ("[*]Searching for Hidden processes by verifying that all thread seen by ps are also seen by others\n\n") ;

	sprintf(command,REVERSE) ;

	// get the path where Unhide is ran from.
	getcwd(curdir, PATH_MAX);

	fich_tmp=popen (command, "r") ;
	if (fich_tmp == NULL) {
		if(TRUE == verbose) {
			printf("Warning : popen failed while running ps (memory, or something set errno: %s)\n", strerror(errno));
		}
		return;
	}

	while (!feof(fich_tmp)) {
		char* curline = read_line;

		fgets(read_line, 1024, fich_tmp);
		read_line[1023] = 0;

//			printf("read_line = %s\n", read_line);   // DEBUG
		while( *curline == ' ' && curline <= read_line+1023) {
			curline++;
		}

		// get LWP
		index=0;
		while( isdigit(*curline) && curline <= read_line+1023) {
			lwp[index++] = *curline;
			curline++;
		}
		lwp[index] = 0; // terminate string

		syspids = -1;
		syspids = atol(lwp);
		if (-1 == syspids) continue ; // something went wrong

		// avoid ourselves
		if (syspids == mypid) {
			continue;
		}

		found=0;
		found_killbefore=0;
		found_killafter=0;

		errno=0;
		ret = kill(syspids, 0);
		if (errno == 0) found_killbefore=1;

		strcpy(directory,"/proc/");
		strcat(directory,lwp);

		statusproc = stat(directory, &buffer) ;
		if (statusproc != 0) {
			found++;
		}

		statusdir = chdir(directory) ;
		if (statusdir != 0) {
			found++;
		}
		else {
			chdir(curdir);
		}

		errno= 0 ;
		ret = getpriority(PRIO_PROCESS, syspids);
		if (errno != 0) found++;

		errno= 0 ;
		ret = getpgid(syspids);
		if (errno != 0) found++;

		errno= 0 ;
		ret = getsid(syspids);
		if (errno != 0) found++;

		errno= 0 ;
		ret = sched_getaffinity(syspids, sizeof(cpu_set_t), &mask);
		if (ret != 0) found++;

		errno= 0 ;
		ret = sched_getparam(syspids, &param);
		if (errno != 0) found++;

		errno= 0 ;
		ret = sched_getscheduler(syspids);
		if (errno != 0) found++;

		errno=0;
		ret = sched_rr_get_interval(syspids, &tp);
		if (errno != 0) found++;

		errno=0;
		ret = kill(syspids, 0);
		if (errno == 0) found_killafter=1;


		/* these should all agree, except if a process went or came in the middle */
		if (found_killbefore == found_killafter) {
			if (0 != found) {
				printf ("Found HIDDEN PID: %i\n", syspids) ;
				printf ("\tCommand = %s\n", curline) ;
					//printf("\tfound = %d\n", found);   // DEBUG
			}
		} /* else: unreliable */
		else {
			if(TRUE == verbose) {
				printf("Warning : reverse test skipped for PID %d", syspids);
			}
		}
	}

	if (fich_tmp != NULL)
		pclose(fich_tmp);

}



void checksysinfo() {

	struct sysinfo info;
	int contador=0;
	int resultado_antes=0;
	int resultado_despues=0;
	int ocultos=0;
	char buffer[500];

	FILE *fich_proceso ;

	printf ("[*]Searching for Hidden processes through sysinfo() scanning\n\n") ;

	sysinfo(&info);
	resultado_antes=info.procs;

	fich_proceso=popen (SYS_COMMAND, "r") ;
	if (fich_proceso == NULL) {
		if(TRUE == verbose) {
			printf("Warning : popen failed while checking sysinfo (memory, or something set errno: %s)\n", strerror(errno));
		}
		return;
	}

	buffer[499] = '\0';
	while (!feof(fich_proceso)) {

//		fscanf( fich_proceso, "%499s", &buffer[0] );
		fgets(buffer, 499, fich_proceso);
		contador++;

	}

	pclose(fich_proceso);

	sysinfo(&info);
	resultado_despues=info.procs;

//	resultado=contador-5;
//	ocultos=info.procs-resultado;
	if (resultado_antes == resultado_despues) {  /* otherwise intermittent activity.. */

		ocultos=resultado_despues - contador + 3;

		if (ocultos != 0) {

			printf("HIDDEN Processes Found: %i\n",abs(ocultos)) ;

		}
	}
	else {
		if(TRUE == verbose) {
			printf("Warning : sysinfo test skipped due to intermittent activity");
		}
	}

}


void brute() {

	int i=0;
	int allpids[maxpid] ;
	int x;
	int y;
	int z;

	printf ("[*]Starting scanning using brute force against PIDS with fork()\n\n") ;

	// PID under 300 are reserved for kernel
	for(x=0; x < 300; x++) {

		allpids[x] = 0 ;
	}


	for(z=300; z < maxpid; z++) {

		allpids[z] = z ;
	}


	for (i=300; i < maxpid; i++) {
		int vpid;
		int status;

		errno= 0 ;

		if ((vpid = vfork()) == 0) {

//		 allpids[getpid()] =  0;

		 _exit(0);
		}

		if (0 == errno) {
			allpids[vpid] =  0;
			waitpid(vpid, &status, 0);
		}
	}

	/* processes that quit at this point in time create false positives */

	for(y=0; y < maxpid; y++) {

		if (allpids[y] != 0) {

			if(!checkps(allpids[y],PS_PROC | PS_THREAD | PS_MORE) ) {

				printbadpid(allpids[y]);

			}
		}

	}


	printf ("[*]Starting scanning using brute force against PIDS with Threads\n\n") ;

	// PID under 300 are reserved for kernel
	for(x=0; x < 300; x++) {

		allpids[x] = 0 ;
	}


	for(z=300; z < maxpid; z++) {

		allpids[z] = z ;
	}


	for (i=300; i < maxpid ; i++) {
		void *status;

		errno= 0 ;

		pthread_t idHilo;

		int error;

		error = pthread_create (&idHilo, NULL, funcionThread, NULL);

		if (error != 0)
		{
			perror ("Warning : Cannot create thread !");
			exit (-1);
		}

		error = pthread_join(idHilo, &status);

		if (error != 0)
		{
			perror ("Warning : Cannot join thread !");
			exit (-1);
		}

		allpids[tid] =  0;

	}

	/* processes that quit at this point in time create false positives */

	for(y=0; y < maxpid; y++) {

		if (allpids[y] != 0) {

			if(!checkps(allpids[y],PS_PROC | PS_THREAD | PS_MORE) ) {

				printbadpid(allpids[y]);

			}
		}

	}

}

void usage(char * command) {

	printf("Usage: %s [options] proc | procfs | sys | brute | reverse | quick\n\n", command);
	printf("Option :\n");
	printf("   -v          verbose\n");
	printf("   -h          display this help\n");
	printf("   -m          more checks (available only with procfs command)\n");
}


int main (int argc, char *argv[]) {

int i, test = TST_NONE;

	printf ("Unhide 20100819\n") ;
	printf ("http://www.security-projects.com/?Unhide\n\n\n") ;
	get_max_pid(&maxpid);

	if(argc < 2) {
		usage(argv[0]);
		exit (1);
	}

	for (i=1 ; i<argc ; i++) {
		if (strcmp(argv[i], "-v") == 0) {verbose = TRUE;}
		else if (strcmp(argv[i], "-h") == 0) {usage(argv[0]); exit(0);}
		else if (strcmp(argv[i], "-m") == 0) {morecheck = TRUE; verbose = TRUE;}
		else if (strcmp(argv[i], "proc") == 0) {test= TST_PROC;}
		else if (strcmp(argv[i], "procfs") == 0) {test= TST_DIR;}
		else if (strcmp(argv[i], "sys") == 0) {test= TST_SYS;}
		else if (strcmp(argv[i], "brute") == 0) {test= TST_BRUTE;}
		else if (strcmp(argv[i], "reverse") == 0) {test= TST_REVERSE;}
		else if (strcmp(argv[i], "quick") == 0) {test= TST_QUICK;}
		else { printf("Unknown argument\n") ; usage(argv[0]); exit(0);}
	}

	setpriority(PRIO_PROCESS,0,-20);  /* reduce risk from intermittent processes - may fail, dont care */

	mypid = getpid();

	switch (test) {
		case TST_PROC :
			checkproc();
			break;
		case TST_DIR :
			checkreaddir();
			checkchdir();
			break;
		case TST_SYS :
			checkkill();
			checkallnoprocps();
			checkgetpriority();
			checkgetpgid() ;
			checkgetsid();
			checksched_getaffinity();
			checksched_getparam();
			checksched_getscheduler();
			checksched_rr_get_interval();
			checksysinfo();
			break;
		case TST_BRUTE :
			brute();
			break;
		case TST_REVERSE :
			checkallreverse();
			break;
		case TST_QUICK :
			checkallquick();
			checksysinfo();
			break;
		default :
			printf("Unknown test\n");
	}



	return 0;
}
